// This modifier takes the styles prepared by the `computeStyles` modifier
// and applies them to the HTMLElements such as popper and arrow
export function applyStyles(state) {
  var data = state.modifiersData.computeStyles;
  Object.keys(state.elements).forEach(function (name) {
    var style = data.styles[name] || {};
    var attributes = data.attributes[name] || {};
    var element = state.elements[name]; // arrow is optional

    if (!element) {
      return;
    }

    Object.assign(element.style, style);
    Object.entries(attributes).forEach(function (args) {
      return element.setAttribute.apply(element, args);
    });
  });
  return state;
}
export function onDestroy(state) {
  var data = state.modifiersData.computeStyles;
  Object.keys(state.elements).forEach(function (name) {
    var element = state.elements[name];
    var styleProperties = Object.keys(data.styles.hasOwnProperty(name) ? Object.assign({}, data.styles[name]) : {});
    var attributes = data.attributes[name] || {}; // Set all values to an empty string to unset them

    var style = styleProperties.reduce(function (style, property) {
      var _Object$assign;

      return Object.assign({}, style, (_Object$assign = {}, _Object$assign[String(property)] = '', _Object$assign));
    }, {}); // arrow is optional

    if (!element) {
      return;
    } // Flow doesn't support to extend this property, but it's the most
    // effective way to apply styles to an HTMLElement
    // $FlowIgnore


    Object.assign(element.style, style);
    Object.keys(attributes).forEach(function (attribute) {
      return element.removeAttribute(attribute);
    });
  });
}
export default {
  name: 'applyStyles',
  enabled: true,
  phase: 'write',
  fn: applyStyles,
  onDestroy: onDestroy,
  requires: ['computeStyles']
};