// This modifier takes the Popper state and prepares some StyleSheet properties
// that can be applied to the popper element to make it render in the expected position.
export var mapStrategyToPosition = function mapStrategyToPosition(strategy) {
  switch (strategy) {
    case 'fixed':
      return 'fixed';

    case 'absolute':
    default:
      return 'absolute';
  }
};
export var computePopperStyles = function computePopperStyles(_ref) {
  var offsets = _ref.offsets,
      strategy = _ref.strategy,
      gpuAcceleration = _ref.gpuAcceleration;

  // @1x displays in Chrome can be blurry due to non-rounded offsets, but this
  // introduces slight positioning errors. TODO: somehow solve this better
  // by default it is active, disable it only if explicitly set to false
  if (gpuAcceleration === false || window.devicePixelRatio < 2) {
    return {
      top: offsets.y + "px",
      left: offsets.x + "px",
      position: mapStrategyToPosition(strategy)
    };
  } else {
    return {
      top: '0px',
      left: '0px',
      transform: "translate3d(" + offsets.x + "px, " + offsets.y + "px, 0)",
      position: mapStrategyToPosition(strategy)
    };
  }
};
export var computeArrowStyles = function computeArrowStyles(_ref2) {
  var offsets = _ref2.offsets,
      gpuAcceleration = _ref2.gpuAcceleration;

  if (gpuAcceleration) {
    return {
      top: offsets.y + "px",
      left: offsets.x + "px",
      position: 'absolute'
    };
  } else {
    return {
      transform: "translate3d(" + offsets.x + "px, " + offsets.y + "px, 0)",
      position: 'absolute'
    };
  }
};
export function computeStyles(state, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$gpuAccelerat = _options.gpuAcceleration,
      gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat;
  state.modifiersData.computeStyles = {
    styles: {
      // popper offsets are always available
      popper: computePopperStyles({
        offsets: state.modifiersData.popperOffsets,
        strategy: state.options.strategy,
        gpuAcceleration: gpuAcceleration
      }),
      // arrow offsets may not be available
      arrow: state.modifiersData.arrow != null ? computeArrowStyles({
        offsets: state.modifiersData.arrow,
        gpuAcceleration: gpuAcceleration
      }) : undefined
    },
    attributes: {
      popper: {
        'data-popper-placement': state.placement
      }
    }
  };
  return state;
}
export default {
  name: 'computeStyles',
  enabled: true,
  phase: 'afterMain',
  fn: computeStyles,
  data: {}
};