import getBoundingClientRect from '../dom-utils/getBoundingClientRect';
import getClippingParent from '../dom-utils/getClippingParent';
import getDocumentRect from '../dom-utils/getDocumentRect';
import getDocumentElement from '../dom-utils/getDocumentElement';
import computeOffsets from '../utils/computeOffsets';
import rectToClientRect from '../utils/rectToClientRect';
export function detectOverflow(state, options) {
  if (options === void 0) {
    options = {
      boundaryElement: getClippingParent(state.elements.popper)
    };
  }

  var popperElement = state.elements.popper;
  var referenceElement = state.elements.reference;
  var popperRect = state.measures.popper;
  var documentElement = getDocumentElement(options.boundaryElement);

  if (!options.boundaryElement.contains(popperElement)) {
    if (process.env.NODE_ENV !== "production") {
      console.error('Popper: "detectOverflow" can accept as `boundaryElement` only a parent node of the provided popper.');
    }

    return state;
  }

  var boundaryClientRect = documentElement === options.boundaryElement ? rectToClientRect(getDocumentRect(documentElement)) : getBoundingClientRect(options.boundaryElement);
  var referenceClientRect = getBoundingClientRect(referenceElement);
  var popperOffsets = computeOffsets({
    reference: referenceClientRect,
    element: popperRect,
    strategy: 'absolute',
    placement: state.placement,
    scroll: {
      scrollTop: 0,
      scrollLeft: 0
    }
  });
  var popperClientRect = rectToClientRect(Object.assign({}, popperRect, {}, popperOffsets));
  state.modifiersData.detectOverflow = {
    top: boundaryClientRect.top - popperClientRect.top,
    bottom: popperClientRect.bottom - boundaryClientRect.bottom,
    left: boundaryClientRect.left - popperClientRect.left,
    right: popperClientRect.right - boundaryClientRect.right
  };
  return state;
}
export default {
  name: 'detectOverflow',
  enabled: true,
  phase: 'read',
  fn: detectOverflow,
  requires: ['popperOffsets'],
  data: {}
};