import { basePlacements, top, left, right, bottom } from '../enums';
import getBasePlacement from '../utils/getBasePlacement';
import getMainAxisFromPlacement from '../utils/getMainAxisFromPlacement';
import getAltAxis from '../utils/getAltAxis';
import mergePaddingObject from '../utils/mergePaddingObject';
import expandToHashMap from '../utils/expandToHashMap';
import within from '../utils/within';
export function preventOverflow(state, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$mainAxis = _options.mainAxis,
      checkMainAxis = _options$mainAxis === void 0 ? true : _options$mainAxis,
      _options$altAxis = _options.altAxis,
      checkAltAxis = _options$altAxis === void 0 ? false : _options$altAxis,
      _options$tether = _options.tether,
      tether = _options$tether === void 0 ? true : _options$tether,
      _options$padding = _options.padding,
      padding = _options$padding === void 0 ? 0 : _options$padding;
  var overflow = state.modifiersData.detectOverflow;
  var basePlacement = getBasePlacement(state.placement);
  var mainAxis = getMainAxisFromPlacement(basePlacement);
  var altAxis = getAltAxis(mainAxis);
  var popperOffsets = state.modifiersData.popperOffsets;
  var referenceRect = state.measures.reference;
  var popperRect = state.measures.popper;
  var paddingObject = mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements));

  if (checkMainAxis) {
    var mainSide = mainAxis === 'y' ? top : left;
    var altSide = mainAxis === 'y' ? bottom : right;
    var len = mainAxis === 'y' ? 'height' : 'width';
    var offset = popperOffsets[mainAxis];
    var min = popperOffsets[mainAxis] + overflow[mainSide] + paddingObject[mainSide];
    var max = popperOffsets[mainAxis] - overflow[altSide] - paddingObject[altSide];
    var tetherMin = state.modifiersData.popperOffsets[mainAxis] - referenceRect[len] / 2 + popperRect[len] / 2;
    var tetherMax = state.modifiersData.popperOffsets[mainAxis] + referenceRect[len] / 2 - popperRect[len] / 2;
    var isReferenceLarger = referenceRect[len] > popperRect[len];
    state.modifiersData.popperOffsets[mainAxis] = within(tether ? Math.min(min, isReferenceLarger ? tetherMax : tetherMin) : min, offset, tether ? Math.max(max, isReferenceLarger ? tetherMin : tetherMax) : max);
  }

  if (checkAltAxis) {
    var _mainSide = mainAxis === 'x' ? top : left;

    var _altSide = mainAxis === 'x' ? bottom : right;

    state.modifiersData.popperOffsets[altAxis] = within(popperOffsets[altAxis] + overflow[_mainSide] + paddingObject[_mainSide], popperOffsets[altAxis], popperOffsets[altAxis] - overflow[_altSide] - paddingObject[_altSide]);
  }

  return state;
}
export default {
  name: 'preventOverflow',
  enabled: true,
  phase: 'main',
  fn: preventOverflow,
  requires: ['detectOverflow']
};