import format from './format';
import { read, main, write } from '../enums';
var ERROR_MESSAGE = 'Popper: modifier "%s" provided an invalid %s property, expected %s but got %s';
var VALID_PROPERTIES = ['name', 'enabled', 'phase', 'fn', 'onLoad', 'requires', 'options'];
export default (function (modifiers) {
  modifiers.forEach(function (modifier) {
    Object.keys(modifier).forEach(function (key) {
      switch (key) {
        case 'name':
          if (typeof modifier.name !== 'string') {
            console.error(format(ERROR_MESSAGE, String(modifier.name), '"name"', '"string"', "\"" + String(modifier.name) + "\""));
          }

          break;

        case 'enabled':
          if (typeof modifier.enabled !== 'boolean') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"enabled"', '"boolean"', "\"" + String(modifier.enabled) + "\""));
          }

        case 'phase':
          if (![read, main, write].includes(modifier.phase)) {
            console.error(format(ERROR_MESSAGE, modifier.name, '"phase"', 'either "read", "main" or "write"', "\"" + String(modifier.phase) + "\""));
          }

          break;

        case 'fn':
          if (typeof modifier.fn !== 'function') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"fn"', '"function"', "\"" + String(modifier.fn) + "\""));
          }

          break;

        case 'onLoad':
          if (typeof modifier.onLoad !== 'function') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"onLoad"', '"function"', "\"" + String(modifier.fn) + "\""));
          }

          break;

        case 'onDestroy':
          if (typeof modifier.onLoad !== 'function') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"onDestroy"', '"function"', "\"" + String(modifier.fn) + "\""));
          }

          break;

        case 'requires':
          if (!Array.isArray(modifier.requires)) {
            console.error(format(ERROR_MESSAGE, modifier.name, '"requires"', '"array"', "\"" + String(modifier.requires) + "\""));
          }

          break;

        case 'options':
        case 'data':
          break;

        default:
          console.error("PopperJS: an invalid property has been provided to the \"" + modifier.name + "\" modifier, valid properties are " + VALID_PROPERTIES.map(function (s) {
            return "\"" + s + "\"";
          }).join(', ') + "; but \"" + key + "\" was provided.");
      }
    });
  });
});